﻿// Based on the WichmannHill algorithm
// but with added randomness based on the current date and time.
package sudoku {
	public class Random {
		private var seed1:int;
		private var seed2:int;
		private var seed3:int;
		
		private const mod1:uint = 30269;
		private const mod2:uint = 30307;
		private const mod3:uint = 30323;
		
		public function Random(s1:int=0, s2:int=0, s3:int=0)
		{
			seed1 = getSeed(s1, mod1);
			seed2 = getSeed(s2, mod2);
			seed3 = getSeed(s3, mod3);
		}
		
		public function next():Number
		{
			seed1 = (seed1 * 171) % mod1;
			seed2 = (seed2 * 172) % mod2;
			seed3 = (seed3 * 170) % mod3;
			
			var r:Number = Number(seed1) / Number(mod1) 
						+ Number(seed2) / Number(mod2) 
						+ Number(seed3) / Number(mod3);
						
			while (r > 1.0) { r -= 1.0; }
			return r;
		}
		
		public function randomize():void
		{
			var date:Date = new Date();
			seed1 = getSeed(date.milliseconds, mod1);
			seed2 = getSeed(date.time, mod2);
		}
		
		private function getSeed(seed:uint, mod:uint):int
		{
			seed = seed % mod;
			while (seed == 0) {
				seed = Math.random()*mod;
			}
			return seed;
		}
	}
}



